# To disable particular handlers, remove the comment and space
#% global without_glfs -Dwith-glfs=FALSE
%global _without_rbd -Dwith-rbd=FALSE
#% global _without_qcow -Dwith-qcow=FALSE
%global _without_zbc -Dwith-zbc=FALSE
#% global _without_fbo -Dwith-fbo=FALSE
%global _without_tcmalloc -Dwith-tcmalloc=FALSE

Name:           tcmu-runner
License:        ASL 2.0 or LGPLv2+
Group:          System Environment/Daemons
Summary:        A daemon that handles the userspace side of the LIO TCM-User backstore
Version:        1.5.3
Release:   	1%{?dev_version:%{dev_version}}%{?dist} 
Vendor:         Oracle America
URL:            https://github.com/open-iscsi/tcmu-runner
Source: 	tcmu-runner-1.5.3.tar.bz2

BuildRequires:  cmake make gcc systemd
BuildRequires:  libnl3-devel glib2-devel zlib-devel kmod-devel

%if 0%{!?_without_glfs:1}
BuildRequires:  libgfapi-devel 
Requires(pre):  libgfapi0 
%endif
%if 0%{!?_without_rbd:1}
BuildRequires:  librbd1-devel librados2-devel
Requires(pre):  librbd1 librados2
%endif
%if 0%{!?_without_tcmalloc:1}
BuildRequires:  gperftools-devel
Requires:       gperftools-libs
%endif

Requires(pre):  kmod zlib libnl3 glib2 logrotate rsyslog targetcli
Requires:       libtcmu = %{version}-%{release}

%description
A daemon that handles the userspace side of the LIO TCM-User backstore.

LIO is the SCSI target in the Linux kernel. It is entirely kernel code, and
allows exported SCSI logical units (LUNs) to be backed by regular files or
block devices. But, if we want to get fancier with the capabilities of the
device we're emulating, the kernel is not necessarily the right place. While
there are userspace libraries for compression, encryption, and clustered
storage solutions like Ceph or Gluster, these are not accessible from the
kernel.

The TCMU userspace-passthrough backstore allows a userspace process to handle
requests to a LUN. But since the kernel-user interface that TCMU provides
must be fast and flexible, it is complex enough that we'd like to avoid each
userspace handler having to write boilerplate code.

tcmu-runner handles the messy details of the TCMU interface -- UIO, netlink,
pthreads, and DBus -- and exports a more friendly C plugin module API. Modules
using this API are called "TCMU handlers". Handler authors can write code just
to handle the SCSI commands as desired, and can also link with whatever
userspace libraries they like.

%if 0%{!?_without_glfs:1}
%package handler-glfs
Summary:        Gluster handler for tcmu-runner
Requires:       %{name} = %{version}-%{release}

%description handler-glfs
A daemon that handles the complexity of the LIO kernel target's userspace
passthrough interface (TCMU). It presents a C plugin API for extension modules
that handle SCSI requests in ways not possible or suitable to be handled
by LIO's in-kernel backstores.

This package contains the Gluster handler.
%endif

%if 0%{!?_without_rbd:1}
%package handler-rbd
Summary:        Ceph RBD handler for tcmu-runner
Requires:       %{name} = %{version}-%{release}

%description handler-rbd
A daemon that handles the complexity of the LIO kernel target's userspace
passthrough interface (TCMU). It presents a C plugin API for extension modules
that handle SCSI requests in ways not possible or suitable to be handled
by LIO's in-kernel backstores.

This package contains the Ceph RBD handler.
%endif

%package -n libtcmu
Summary:       A library supporting LIO TCM-User backstores processing
Group:         Development/Libraries

%description -n libtcmu
libtcmu provides a library for processing SCSI commands exposed by the
LIO kernel target's TCM-User backend.

%package -n libtcmu-devel
Summary:        Development headers for libtcmu
Group:          Development/Libraries 
Requires:       %{name} = %{version}-%{release}
Requires:       libtcmu = %{version}-%{release}

%description -n libtcmu-devel
Development header(s) for developing against libtcmu.

%global debug_package %{nil}

%prep
%setup -q -n %{name}-%{version}%{?candidate:-%{candidate}}

%build
%cmake -DSUPPORT_SYSTEMD=ON \
%{?_without_glfs} \
%{?_without_rbd} \
%{?_without_zbc} \
%{?_without_qcow} \
%{?_without_fbo} \
%{?_without_tcmalloc} .
make %{?_smp_mflags}

%install
make install DESTDIR=%{buildroot}
%{__rm} -f %{buildroot}/etc/tcmu/tcmu.conf.old
%{__rm} -f %{buildroot}/etc/logrotate.d/tcmu-runner.bak/tcmu-runner
%{__install} -m755 -d %{buildroot}/%{_includedir}/
%{__install} -m644 libtcmu.h %{buildroot}/%{_includedir}/libtcmu.h
%{__install} -m644 libtcmu_common.h %{buildroot}/%{_includedir}/libtcmu_common.h

%post -n libtcmu -p /sbin/ldconfig

%postun -n libtcmu -p /sbin/ldconfig

%files
%{_bindir}/tcmu-runner
%dir %{_sysconfdir}/dbus-1/
%dir %{_sysconfdir}/dbus-1/system.d
%config %{_sysconfdir}/dbus-1/system.d/tcmu-runner.conf
%dir %{_datadir}/dbus-1/
%dir %{_datadir}/dbus-1/system-services/
%{_datadir}/dbus-1/system-services/org.kernel.TCMUService1.service
%{_unitdir}/tcmu-runner.service
%dir %{_libdir}/tcmu-runner/
%{_libdir}/tcmu-runner/*.so
%{_mandir}/man8/*
%doc README.md LICENSE.LGPLv2.1 LICENSE.Apache2
%dir %{_sysconfdir}/tcmu/
%config %{_sysconfdir}/tcmu/tcmu.conf
%config(noreplace) %{_sysconfdir}/logrotate.d/tcmu-runner
%ghost %attr(0644,-,-) %{_sysconfdir}/tcmu/tcmu.conf.old
%ghost %attr(0644,-,-) %{_sysconfdir}/logrotate.d/tcmu-runner.bak/tcmu-runner


%if 0%{!?_without_glfs:1}
%files handler-glfs
%{_libdir}/tcmu-runner/handler_glfs.so
%endif

%if 0%{!?_without_rbd:1}
%files handler-rbd
%{_libdir}/tcmu-runner/handler_rbd.so
%endif

%files -n libtcmu
%{_libdir}/libtcmu*.so.*

%files -n libtcmu-devel
%{_includedir}/libtcmu.h
%{_includedir}/libtcmu_common.h
%{_libdir}/libtcmu*.so

%changelog
* Fri Apr 16 2021 Frank Batschulat <frank.batschulat@oracle.com> - 1.5.3-1
- Initial spec file for oracle-gluster-8
